@@ -21,7 +21,9 @@
  *
  **************************************************************************/
 
-#include "config.h"
+#include "nm-default.h"
+
+#include "auth-helpers.h"
 
 #include <string.h>
 #include <sys/types.h>
@@ -31,11 +33,6 @@
 #include <unistd.h>
 #include <errno.h>
 
-#include "nm-default.h"
-
-#include "auth-helpers.h"
-#include "nm-openvpn.h"
-#include "nm-openvpn-service-defines.h"
 #include "utils.h"
 
 #define BLOCK_HANDLER_ID "block-handler-id"
@@ -87,8 +84,8 @@ tls_cert_changed_cb (GtkWidget *widget, gpointer data)
 {
 	GtkWidget *other_widgets[2] = { ((TlsChooserSignalData *) data)->widget1,
 	                                ((TlsChooserSignalData *) data)->widget2 };
-	GtkFileChooser *this, *others[2];
-	char *fname, *other_fnames[2];
+	GtkFileChooser *this = GTK_FILE_CHOOSER (widget);
+	char *fname, *dirname, *tmp;
 	int i;
 
 	/* If the just-changed file chooser is a PKCS#12 file, then all of the
@@ -100,43 +97,38 @@ tls_cert_changed_cb (GtkWidget *widget, gpointer data)
 	 * connection (CA, client cert, private key).
 	 */
 
-	this = GTK_FILE_CHOOSER (widget);
-	others[0] = GTK_FILE_CHOOSER (other_widgets[0]);
-	others[1] = GTK_FILE_CHOOSER (other_widgets[1]);
-
 	fname = gtk_file_chooser_get_filename (this);
-	other_fnames[0] = gtk_file_chooser_get_filename (others[0]);
-	other_fnames[1] = gtk_file_chooser_get_filename (others[1]);
+	dirname = g_path_get_dirname (fname);
 
-	if (is_pkcs12 (fname)) {
-		/* Make sure all choosers have this PKCS#12 file */
-		for (i = 0; i < 2; i++) {
-			if (!other_fnames[i] || strcmp (fname, other_fnames[i])) {
-				/* Next chooser was different, make it the same as the first */
-				gulong id = GPOINTER_TO_SIZE (g_object_get_data (G_OBJECT (other_widgets[i]),
-				                                                 BLOCK_HANDLER_ID));
-				g_signal_handler_block (other_widgets[i], id);
-				gtk_file_chooser_set_filename (others[i], fname);
-				g_signal_handler_unblock (other_widgets[i], id);
-			}
-		}
-	} else {
-		/* Just-chosen file isn't PKCS#12 or no file was chosen, so clear out other
-		 * file selectors that have PKCS#12 files in them.
-		 */
-		for (i = 0; i < 2; i++) {
-			if (is_pkcs12 (other_fnames[i])) {
-				gulong id = GPOINTER_TO_SIZE (g_object_get_data (G_OBJECT (other_widgets[i]),
-				                                                 BLOCK_HANDLER_ID));
-				g_signal_handler_block (other_widgets[i], id);
-				gtk_file_chooser_unselect_all (others[i]);
-				g_signal_handler_unblock (other_widgets[i], id);
-			}
+	for (i = 0; i < G_N_ELEMENTS (other_widgets); i++) {
+		GtkFileChooser *other = GTK_FILE_CHOOSER (other_widgets[i]);
+		char *other_fname = gtk_file_chooser_get_filename (other);
+		gulong id = GPOINTER_TO_SIZE (g_object_get_data (G_OBJECT (other), BLOCK_HANDLER_ID));
+
+		g_signal_handler_block (other, id);
+		if (is_pkcs12 (fname)) {
+			/* Make sure all choosers have this PKCS#12 file */
+			if (!other_fname || strcmp (fname, other_fname))
+				gtk_file_chooser_set_filename (other, fname);
+		} else {
+			/* Just-chosen file isn't PKCS#12 or no file was chosen, so clear out other
+			 * file selectors that have PKCS#12 files in them.
+			 */
+			if (is_pkcs12 (other_fname))
+				gtk_file_chooser_unselect_all (other);
+
+			/* Set directory of un-set file choosers to the directory just selected */
+			tmp = gtk_file_chooser_get_filename (other);
+			if (!tmp && dirname)
+				gtk_file_chooser_set_current_folder (other, dirname);
+			g_free (tmp);
 		}
+		g_signal_handler_unblock (other, id);
+		g_free (other_fname);
 	}
+
 	g_free (fname);
-	g_free (other_fnames[0]);
-	g_free (other_fnames[1]);
+	g_free (dirname);
 }
 
 static void
@@ -433,8 +425,8 @@ validate_tls (GtkBuilder *builder, const char *prefix, GError **error)
 	g_free (tmp);
 	if (!valid) {
 		g_set_error (error,
-		             OPENVPN_EDITOR_PLUGIN_ERROR,
-		             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
+		             NMV_EDITOR_PLUGIN_ERROR,
+		             NMV_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 		             NM_OPENVPN_KEY_CA);
 		return FALSE;
 	}
@@ -444,8 +436,8 @@ validate_tls (GtkBuilder *builder, const char *prefix, GError **error)
 	g_free (tmp);
 	if (!valid) {
 		g_set_error (error,
-		             OPENVPN_EDITOR_PLUGIN_ERROR,
-		             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
+		             NMV_EDITOR_PLUGIN_ERROR,
+		             NMV_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 		             NM_OPENVPN_KEY_CERT);
 		return FALSE;
 	}
@@ -456,8 +448,8 @@ validate_tls (GtkBuilder *builder, const char *prefix, GError **error)
 	g_free (tmp);
 	if (!valid) {
 		g_set_error (error,
-		             OPENVPN_EDITOR_PLUGIN_ERROR,
-		             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
+		             NMV_EDITOR_PLUGIN_ERROR,
+		             NMV_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 		             NM_OPENVPN_KEY_KEY);
 		return FALSE;
 	}
@@ -478,8 +470,8 @@ validate_tls (GtkBuilder *builder, const char *prefix, GError **error)
 	if (encrypted && secrets_required) {
 		if (!gtk_entry_get_text_length (GTK_ENTRY (widget))) {
 			g_set_error (error,
-			             OPENVPN_EDITOR_PLUGIN_ERROR,
-			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
+			             NMV_EDITOR_PLUGIN_ERROR,
+			             NMV_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_CERTPASS);
 			return FALSE;
 		}
@@ -505,16 +497,16 @@ auth_widget_check_validity (GtkBuilder *builder, const char *contype, GError **e
 		str = gtk_entry_get_text (GTK_ENTRY (widget));
 		if (!str || !strlen (str)) {
 			g_set_error (error,
-			             OPENVPN_EDITOR_PLUGIN_ERROR,
-			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
+			             NMV_EDITOR_PLUGIN_ERROR,
+			             NMV_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_USERNAME);
 			return FALSE;
 		}
 	} else if (!strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD)) {
 		if (!validate_file_chooser (builder, "pw_ca_cert_chooser")) {
 			g_set_error (error,
-			             OPENVPN_EDITOR_PLUGIN_ERROR,
-			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
+			             NMV_EDITOR_PLUGIN_ERROR,
+			             NMV_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_CA);
 			return FALSE;
 		}
@@ -522,16 +514,16 @@ auth_widget_check_validity (GtkBuilder *builder, const char *contype, GError **e
 		str = gtk_entry_get_text (GTK_ENTRY (widget));
 		if (!str || !strlen (str)) {
 			g_set_error (error,
-			             OPENVPN_EDITOR_PLUGIN_ERROR,
-			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
+			             NMV_EDITOR_PLUGIN_ERROR,
+			             NMV_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_USERNAME);
 			return FALSE;
 		}
 	} else if (!strcmp (contype, NM_OPENVPN_CONTYPE_STATIC_KEY)) {
 		if (!validate_file_chooser (builder, "sk_key_chooser")) {
 			g_set_error (error,
-			             OPENVPN_EDITOR_PLUGIN_ERROR,
-			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
+			             NMV_EDITOR_PLUGIN_ERROR,
+			             NMV_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_STATIC_KEY);
 			return FALSE;
 		}
@@ -540,8 +532,8 @@ auth_widget_check_validity (GtkBuilder *builder, const char *contype, GError **e
 		str = gtk_entry_get_text (GTK_ENTRY (widget));
 		if (!str || !strlen (str)) {
 			g_set_error (error,
-			             OPENVPN_EDITOR_PLUGIN_ERROR,
-			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
+			             NMV_EDITOR_PLUGIN_ERROR,
+			             NMV_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_LOCAL_IP);
 			return FALSE;
 		}
@@ -550,8 +542,8 @@ auth_widget_check_validity (GtkBuilder *builder, const char *contype, GError **e
 		str = gtk_entry_get_text (GTK_ENTRY (widget));
 		if (!str || !strlen (str)) {
 			g_set_error (error,
-			             OPENVPN_EDITOR_PLUGIN_ERROR,
-			             OPENVPN_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
+			             NMV_EDITOR_PLUGIN_ERROR,
+			             NMV_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 			             NM_OPENVPN_KEY_REMOTE_IP);
 			return FALSE;
 		}
@@ -899,6 +891,7 @@ static const char *advanced_keys[] = {
 	NM_OPENVPN_KEY_REMOTE_RANDOM,
 	NM_OPENVPN_KEY_TUN_IPV6,
 	NM_OPENVPN_KEY_REMOTE_CERT_TLS,
+	NM_OPENVPN_KEY_NS_CERT_TYPE,
 	NM_OPENVPN_KEY_PING,
 	NM_OPENVPN_KEY_PING_EXIT,
 	NM_OPENVPN_KEY_PING_RESTART,
@@ -1210,6 +1203,53 @@ tls_auth_toggled_cb (GtkWidget *widget, gpointer user_data)
 	gtk_widget_set_sensitive (widget, use_auth);
 }
 
+static void
+ns_cert_type_toggled_cb (GtkWidget *widget, gpointer user_data)
+{
+	GtkBuilder *builder = (GtkBuilder *) user_data;
+	gboolean use_ns_cert_type = FALSE;
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ns_cert_type_checkbutton"));
+	use_ns_cert_type = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ns_cert_type_label"));
+	gtk_widget_set_sensitive (widget, use_ns_cert_type);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ns_cert_type_combo"));
+	gtk_widget_set_sensitive (widget, use_ns_cert_type);
+}
+
+#define NS_CERT_TYPE_COL_NAME 0
+#define NS_CERT_TYPE_COL_VALUE 1
+
+static void
+populate_ns_cert_type_combo (GtkComboBox *box, const char *type)
+{
+	GtkListStore *store;
+	GtkTreeIter iter;
+
+	store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+	gtk_combo_box_set_model (box, GTK_TREE_MODEL (store));
+
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter,
+	                    NS_CERT_TYPE_COL_NAME, _("Server"),
+	                    NS_CERT_TYPE_COL_VALUE, NM_OPENVPN_NS_CERT_TYPE_SERVER,
+	                    -1);
+	gtk_list_store_append (store, &iter);
+	gtk_list_store_set (store, &iter,
+	                    NS_CERT_TYPE_COL_NAME, _("Client"),
+	                    NS_CERT_TYPE_COL_VALUE, NM_OPENVPN_NS_CERT_TYPE_CLIENT,
+	                    -1);
+
+	if (g_strcmp0 (type, NM_OPENVPN_NS_CERT_TYPE_CLIENT) == 0)
+		gtk_combo_box_set_active (box, 1);
+	else
+		gtk_combo_box_set_active (box, 0);
+
+	g_object_unref (store);
+}
+
 #define PROXY_TYPE_NONE  0
 #define PROXY_TYPE_HTTP  1
 #define PROXY_TYPE_SOCKS 2
@@ -1327,10 +1367,10 @@ device_name_changed_cb (GtkEntry *entry,
 	/* Change cell's background to red if the value is invalid */
 	if (entry_sensitive && entry_text[0] != '\0' && !nm_utils_iface_valid_name (entry_text)) {
 		gdk_rgba_parse (&rgba, "red");
-		gtk_widget_override_background_color (GTK_WIDGET (editable), GTK_STATE_NORMAL, &rgba);
+		gtk_widget_override_background_color (GTK_WIDGET (editable), GTK_STATE_FLAG_NORMAL, &rgba);
 		gtk_widget_set_sensitive (ok_button, FALSE);
 	} else {
-		gtk_widget_override_background_color (GTK_WIDGET (editable), GTK_STATE_NORMAL, NULL);
+		gtk_widget_override_background_color (GTK_WIDGET (editable), GTK_STATE_FLAG_NORMAL, NULL);
 		gtk_widget_set_sensitive (ok_button, TRUE);
 	}
 
@@ -1594,7 +1634,12 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 	}
 
 	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_COMP_LZO);
-	if (value && !strcmp (value, "yes")) {
+	if (NM_IN_STRSET (value, "yes", "adaptive")) {
+		/* the UI currently only supports "--comp-lzo yes" or omitting the "--comp-lzo"
+		 * flag.
+		 *
+		 * Internally, we also support "--comp-lzo [adaptive]" and "--comp-lzo no"
+		 * which have different meaning for openvpn. */
 		widget = GTK_WIDGET (gtk_builder_get_object (builder, "lzo_checkbutton"));
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 	}
@@ -1713,6 +1758,17 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_REMOTE_CERT_TLS);
 	populate_remote_cert_tls_combo (GTK_COMBO_BOX (widget), value);
 
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ns_cert_type_checkbutton"));
+	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_NS_CERT_TYPE);
+	if (value && strlen (value))
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
+	g_signal_connect (G_OBJECT (widget), "toggled", G_CALLBACK (ns_cert_type_toggled_cb), builder);
+	ns_cert_type_toggled_cb (widget, builder);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ns_cert_type_combo"));
+	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_NS_CERT_TYPE);
+	populate_ns_cert_type_combo (GTK_COMBO_BOX (widget), value);
+
 	if (   !strcmp (contype, NM_OPENVPN_CONTYPE_TLS)
 	    || !strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD_TLS)
 	    || !strcmp (contype, NM_OPENVPN_CONTYPE_PASSWORD)) {
@@ -1951,8 +2007,12 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 	}
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "lzo_checkbutton"));
-	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
+		/* we only have a checkbox, which we either map to "--comp-lzo yes" or
+		 * no "--comp-lzo" flag. In the UI, we cannot express "--comp-lzo [adaptive]"
+		 * or "--comp-lzo no". */
 		g_hash_table_insert (hash, g_strdup (NM_OPENVPN_KEY_COMP_LZO), g_strdup ("yes"));
+	}
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "mssfix_checkbutton"));
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
@@ -2052,6 +2112,21 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 			}
 		}
 
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ns_cert_type_checkbutton"));
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
+			widget = GTK_WIDGET (gtk_builder_get_object (builder, "ns_cert_type_combo"));
+			model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
+			if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (widget), &iter)) {
+				char *type = NULL;
+
+				gtk_tree_model_get (model, &iter, NS_CERT_TYPE_COL_VALUE, &type, -1);
+				if (type)
+					g_hash_table_insert (hash,
+					                     g_strdup (NM_OPENVPN_KEY_NS_CERT_TYPE),
+					                     type);
+			}
+		}
+
 		widget = GTK_WIDGET (gtk_builder_get_object (builder, "tls_auth_checkbutton"));
 		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
 			char *filename;
