@@ -19,7 +19,9 @@
  *
  **************************************************************************/
 
-#include "config.h"
+#include "nm-default.h"
+
+#include "import-export.h"
 
 #include <string.h>
 #include <sys/types.h>
@@ -32,14 +34,8 @@
 #include <ctype.h>
 #include <stdio.h>
 
-#include "nm-default.h"
-
-#include "import-export.h"
-#include "nm-openvpn.h"
-#include "nm-openvpn-service-defines.h"
 #include "utils.h"
-#include "nm-macros-internal.h"
-#include "nm-shared-utils.h"
+#include "nm-utils/nm-shared-utils.h"
 
 
 #define INLINE_BLOB_CA                  "ca"
@@ -284,10 +280,12 @@ args_params_parse_port (const char **params, guint n_param, gint64 *out, char **
 static gboolean
 args_params_parse_ip4 (const char **params,
                        guint n_param,
+                       gboolean ovpn_extended_format,
                        in_addr_t *out,
                        char **out_error)
 {
 	in_addr_t a;
+	const char *p;
 
 	g_return_val_if_fail (params, FALSE);
 	g_return_val_if_fail (params[0], FALSE);
@@ -296,14 +294,44 @@ args_params_parse_ip4 (const char **params,
 	g_return_val_if_fail (out, FALSE);
 	g_return_val_if_fail (out_error && !*out_error, FALSE);
 
-	if (inet_pton (AF_INET, params[n_param], &a) != 1) {
-		*out_error = g_strdup_printf (_("invalid %uth argument to '%s' where IPv4 address expected"),
+	if (inet_pton (AF_INET, params[n_param], &a) == 1) {
+		*out = a;
+		return TRUE;
+	}
+
+	if (   ovpn_extended_format
+	    && NM_IN_STRSET (params[n_param], "vpn_gateway", "net_gateway", "remote_host")) {
+		/* we don't support these special destinations, as they currently cannot be expressed
+		 * in a connection. */
+		*out_error = g_strdup_printf (_("unsupported %uth argument %s to '%s'"),
+		                              n_param,
+		                              params[n_param],
+		                              params[0]);
+		return FALSE;
+	}
+
+	if (   ovpn_extended_format
+	    && params[n_param]
+	    && strlen (params[n_param]) <= 255) {
+		for (p = params[n_param]; *p; p++) {
+			if (NM_IN_SET (*p, '-', '.'))
+				continue;
+			if (g_ascii_isalnum (*p))
+				continue;
+			goto not_dns;
+		}
+		/* we also don't support specifing a FQDN. */
+		*out_error = g_strdup_printf (_("unsupported %uth argument to '%s' which looks like a FQDN but only IPv4 address supported"),
 		                              n_param,
 		                              params[0]);
 		return FALSE;
 	}
-	*out = a;
-	return TRUE;
+
+not_dns:
+	*out_error = g_strdup_printf (_("invalid %uth argument to '%s' where IPv4 address expected"),
+	                              n_param,
+	                              params[0]);
+	return FALSE;
 }
 
 static gboolean
@@ -431,7 +459,7 @@ args_parse_line (const char *line,
 
 	index = g_array_new (FALSE, FALSE, sizeof (gsize));
 
-	do {
+	for (;;) {
 		char quote, ch0;
 		gssize word_start = line - line_start;
 		gsize index_i;
@@ -439,51 +467,66 @@ args_parse_line (const char *line,
 		index_i = str_buf - str_buf_orig;
 		g_array_append_val (index, index_i);
 
-		do {
-			switch ((ch0 = _ch_step_1 (&line, &line_len))) {
-			case '"':
-			case '\'':
-				quote = ch0;
+		switch ((ch0 = _ch_step_1 (&line, &line_len))) {
+		case '"':
+		case '\'':
+			quote = ch0;
 
-				while (line_len > 0 && line[0] != quote) {
-					if (quote == '"' && line[0] == '\\') {
-						_ch_step_1 (&line, &line_len);
-						if (line_len <= 0)
-							break;
-					}
-					_strbuf_append_c (&str_buf, &str_buf_len, _ch_step_1 (&line, &line_len));
-				}
-
-				if (line_len <= 0) {
-					*out_error = g_strdup_printf (_("unterminated %s at position %lld"),
-					                              quote == '"' ? _("double quote") : _("single quote"),
-					                              (long long) word_start);
-					return FALSE;
-				}
-
-				_ch_step_1 (&line, &line_len);
-				break;
-			case '\\':
-				if (line_len <= 0) {
-					*out_error = g_strdup_printf (_("trailing escaping backslash at position %lld"),
-					                              (long long) word_start);
-					return FALSE;
+			while (line_len > 0 && line[0] != quote) {
+				if (quote == '"' && line[0] == '\\') {
+					_ch_step_1 (&line, &line_len);
+					if (line_len <= 0)
+						break;
 				}
 				_strbuf_append_c (&str_buf, &str_buf_len, _ch_step_1 (&line, &line_len));
-				break;
-			default:
-				if (g_ascii_isspace (ch0))
-					goto word_completed;
-				_strbuf_append_c (&str_buf, &str_buf_len, ch0);
-				break;
 			}
-		} while (line_len > 0);
-word_completed:
+
+			if (line_len <= 0) {
+				*out_error = g_strdup_printf (_("unterminated %s at position %lld"),
+				                              quote == '"' ? _("double quote") : _("single quote"),
+				                              (long long) word_start);
+				return FALSE;
+			}
+
+			/* openvpn terminates parsing of quoted paramaters after the closing quote.
+			 * E.g. "'a'b" gives "a", "b". */
+			_ch_step_1 (&line, &line_len);
+			break;
+		default:
+			/* once openvpn encounters a non-quoted word, it doesn't consider quoting
+			 * inside the word.
+			 * E.g. "a'b'" gives "a'b'". */
+			for (;;) {
+				if (ch0 == '\\') {
+					if (line_len <= 0) {
+						*out_error = g_strdup_printf (_("trailing escaping backslash at position %lld"),
+						                              (long long) word_start);
+						return FALSE;
+					}
+					ch0 = _ch_step_1 (&line, &line_len);
+				}
+				_strbuf_append_c (&str_buf, &str_buf_len, ch0);
+				if (line_len <= 0)
+					break;
+				ch0 = _ch_step_1 (&line, &line_len);
+				if (g_ascii_isspace (ch0))
+					break;
+			}
+			break;
+		}
 
 		/* the current word is complete.*/
 		_strbuf_append_c (&str_buf, &str_buf_len, '\0');
 		_ch_skip_over_leading_whitespace (&line, &line_len);
-	} while (line_len > 0);
+
+		if (line_len <= 0)
+			break;
+
+		if (NM_IN_SET (line[0], ';', '#')) {
+			/* comments are allowed to start at the beginning of the next word. */
+			break;
+		}
+	}
 
 	str_buf_len = str_buf - str_buf_orig;
 
@@ -700,8 +743,8 @@ inline_blob_write_out (const InlineBlobData *data, GError **error)
 		/* in test mode we don't create the certificate directory. */
 		if (!inline_blob_mkdir_parents (data, data->path, &err_msg)) {
 			g_set_error (error,
-			             OPENVPN_EDITOR_PLUGIN_ERROR,
-			             OPENVPN_EDITOR_PLUGIN_ERROR_FAILED,
+			             NMV_EDITOR_PLUGIN_ERROR,
+			             NMV_EDITOR_PLUGIN_ERROR_FAILED,
 			             _("cannot write <%s> blob from line %ld to file (%s)"),
 			             data->token,
 			             (long) data->token_start_line,
@@ -717,8 +760,8 @@ inline_blob_write_out (const InlineBlobData *data, GError **error)
 	 * access the file later on is left as exercise for the user. */
 	if (!g_file_set_contents (data->path, data->blob_data->str, data->blob_data->len, NULL)) {
 		g_set_error (error,
-		             OPENVPN_EDITOR_PLUGIN_ERROR,
-		             OPENVPN_EDITOR_PLUGIN_ERROR_FAILED,
+		             NMV_EDITOR_PLUGIN_ERROR,
+		             NMV_EDITOR_PLUGIN_ERROR_FAILED,
 		             _("cannot write <%s> blob from line %ld to file '%s'"),
 		             data->token,
 		             (long) data->token_start_line,
@@ -910,15 +953,11 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_COMP_LZO)) {
 			if (!args_params_check_nargs_minmax (params, 0, 1, &line_error))
 				goto handle_line_error;
-			if (params[1]) {
-				if (nm_streq (params[1], "no"))
-					continue;
-				if (!nm_streq (params[1], "yes")) {
-					line_error = g_strdup_printf (_("unsupported comp-lzo argument"));
-					goto handle_line_error;
-				}
+			if (!NM_IN_STRSET (params[1], NULL, "no", "yes", "adaptive")) {
+				line_error = g_strdup_printf (_("unsupported comp-lzo argument"));
+				goto handle_line_error;
 			}
-			setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, "yes");
+			setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, params[1] ?: "adaptive");
 			continue;
 		}
 
@@ -1152,6 +1191,15 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			continue;
 		}
 
+		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_TLS_CIPHER)) {
+			if (!args_params_check_nargs_n (params, 1, &line_error))
+				goto handle_line_error;
+			if (!args_params_check_arg_utf8 (params, 1, NULL, &line_error))
+				goto handle_line_error;
+			setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_TLS_CIPHER, params[1]);
+			continue;
+		}
+
 		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_KEEPALIVE)) {
 			gint64 v2;
 
@@ -1232,18 +1280,18 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			if (!args_params_check_nargs_minmax (params, 1, 4, &line_error))
 				goto handle_line_error;
 
-			if (!args_params_parse_ip4 (params, 1, &network, &line_error))
+			if (!args_params_parse_ip4 (params, 1, TRUE, &network, &line_error))
 				goto handle_line_error;
 
 			if (params[2]) {
 				in_addr_t netmask;
 
-				if (!args_params_parse_ip4 (params, 2, &netmask, &line_error))
+				if (!args_params_parse_ip4 (params, 2, FALSE, &netmask, &line_error))
 					goto handle_line_error;
 				prefix = nm_utils_ip4_netmask_to_prefix (netmask);
 
 				if (params[3]) {
-					if (!args_params_parse_ip4 (params, 3, &gateway, &line_error))
+					if (!args_params_parse_ip4 (params, 3, TRUE, &gateway, &line_error))
 						goto handle_line_error;
 					if (params[4]) {
 						if (!args_params_parse_int64 (params, 4, 0, G_MAXUINT32, &v_int64, &line_error))
@@ -1253,8 +1301,18 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 				}
 			}
 
+			if (prefix == 0 && network == 0) {
+				/* the default-route cannot be specified as normal route in NMSettingIPConfig.
+				 * Just set never-default=FALSE (which is already the default). */
+				g_object_set (s_ip4,
+				              NM_SETTING_IP_CONFIG_NEVER_DEFAULT,
+				              FALSE,
+				              NULL);
+				continue;
+			}
+
 			{
-#ifdef NM_OPENVPN_OLD
+#ifdef NM_VPN_OLD
 				NMIP4Route *route;
 
 				route = nm_ip4_route_new ();
@@ -1378,8 +1436,8 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 
 handle_line_error:
 		g_set_error (error,
-		             OPENVPN_EDITOR_PLUGIN_ERROR,
-		             OPENVPN_EDITOR_PLUGIN_ERROR_FILE_INVALID,
+		             NMV_EDITOR_PLUGIN_ERROR,
+		             NMV_EDITOR_PLUGIN_ERROR_FILE_INVALID,
 		             _("configuration error: %s (line %ld)"),
 		             line_error ? : _("unknown or unsupported option"),
 		             (long) contents_cur_line);
@@ -1389,16 +1447,16 @@ handle_line_error:
 
 	if (!have_client && !have_sk) {
 		g_set_error_literal (error,
-		                     OPENVPN_EDITOR_PLUGIN_ERROR,
-		                     OPENVPN_EDITOR_PLUGIN_ERROR_FILE_NOT_OPENVPN,
+		                     NMV_EDITOR_PLUGIN_ERROR,
+		                     NMV_EDITOR_PLUGIN_ERROR_FILE_NOT_VPN,
 		                     _("The file to import wasn't a valid OpenVPN client configuration"));
 		goto out_error;
 	}
 
 	if (!have_remote) {
 		g_set_error_literal (error,
-		                     OPENVPN_EDITOR_PLUGIN_ERROR,
-		                     OPENVPN_EDITOR_PLUGIN_ERROR_FILE_NOT_OPENVPN,
+		                     NMV_EDITOR_PLUGIN_ERROR,
+		                     NMV_EDITOR_PLUGIN_ERROR_FILE_NOT_VPN,
 		                     _("The file to import wasn't a valid OpenVPN configure (no remote)"));
 		goto out_error;
 	}
@@ -1626,12 +1684,11 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 	const char *proxy_type = NULL;
 	guint i, num;
 	nm_auto(_auto_free_gstring_p) GString *f = NULL;
-	gs_free_error GError *local = NULL;
 
 	if (!path || !path[0]) {
 		g_set_error_literal (error,
-		                     OPENVPN_EDITOR_PLUGIN_ERROR,
-		                     OPENVPN_EDITOR_PLUGIN_ERROR_FILE_NOT_OPENVPN,
+		                     NMV_EDITOR_PLUGIN_ERROR,
+		                     NMV_EDITOR_PLUGIN_ERROR_FILE_NOT_VPN,
 		                     _("missing path argument"));
 		return NULL;
 	}
@@ -1640,8 +1697,8 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 	s_vpn = nm_connection_get_setting_vpn (connection);
 	if (!s_con || !s_vpn) {
 		g_set_error_literal (error,
-		                     OPENVPN_EDITOR_PLUGIN_ERROR,
-		                     OPENVPN_EDITOR_PLUGIN_ERROR_FILE_NOT_OPENVPN,
+		                     NMV_EDITOR_PLUGIN_ERROR,
+		                     NMV_EDITOR_PLUGIN_ERROR_FILE_NOT_VPN,
 		                     _("connection is not a valid OpenVPN connection"));
 		return NULL;
 	}
@@ -1649,8 +1706,8 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 	gateways = _arg_is_set (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE));
 	if (!gateways) {
 		g_set_error_literal (error,
-		                     OPENVPN_EDITOR_PLUGIN_ERROR,
-		                     OPENVPN_EDITOR_PLUGIN_ERROR_FILE_NOT_OPENVPN,
+		                     NMV_EDITOR_PLUGIN_ERROR,
+		                     NMV_EDITOR_PLUGIN_ERROR_FILE_NOT_VPN,
 		                     _("connection was incomplete (missing gateway)"));
 		return NULL;
 	}
@@ -1754,10 +1811,13 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 
 	args_write_line_setting_value (f, NMV_OVPN_TAG_CIPHER, s_vpn, NM_OPENVPN_KEY_CIPHER);
 
+	args_write_line_setting_value (f, NMV_OVPN_TAG_TLS_CIPHER, s_vpn, NM_OPENVPN_KEY_TLS_CIPHER);
+
 	args_write_line_setting_value_int (f, NMV_OVPN_TAG_KEYSIZE, s_vpn, NM_OPENVPN_KEY_KEYSIZE);
 
-	if (nm_streq0 (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO), "yes"))
-		args_write_line (f, NMV_OVPN_TAG_COMP_LZO, "yes");
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO);
+	if (value)
+		args_write_line (f, NMV_OVPN_TAG_COMP_LZO, value);
 
 	if (nm_streq0 (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_FLOAT), "yes"))
 		args_write_line (f, NMV_OVPN_TAG_FLOAT);
@@ -1877,7 +1937,7 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 
 	s_ip4 = nm_connection_get_setting_ip4_config (connection);
 	if (s_ip4) {
-#ifdef NM_OPENVPN_OLD
+#ifdef NM_VPN_OLD
 		num = nm_setting_ip4_config_get_num_routes (s_ip4);
 #else
 		num = nm_setting_ip_config_get_num_routes (s_ip4);
@@ -1890,7 +1950,7 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 			guint64 metric;
 			char metric_buf[50];
 
-#ifdef NM_OPENVPN_OLD
+#ifdef NM_VPN_OLD
 			char next_hop_str_buf[INET_ADDRSTRLEN] = { 0 };
 			char dest_str_buf[INET_ADDRSTRLEN] = { 0 };
 			in_addr_t dest, next_hop;
@@ -1932,10 +1992,10 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 	args_write_line (f, NMV_OVPN_TAG_SCRIPT_SECURITY, "2");
 	args_write_line (f, NMV_OVPN_TAG_PERSIST_KEY);
 	args_write_line (f, NMV_OVPN_TAG_PERSIST_TUN);
-	args_write_line (f, NMV_OVPN_TAG_USER, "openvpn");
-	args_write_line (f, NMV_OVPN_TAG_GROUP, "openvpn");
+	args_write_line (f, NMV_OVPN_TAG_USER, NM_OPENVPN_USER);
+	args_write_line (f, NMV_OVPN_TAG_GROUP, NM_OPENVPN_GROUP);
 
-	return nm_unauto (&f);
+	return g_steal_pointer (&f);
 }
 
 gboolean
@@ -1950,8 +2010,8 @@ do_export (const char *path, NMConnection *connection, GError **error)
 
 	if (!g_file_set_contents (path, f->str, f->len, &local)) {
 		g_set_error (error,
-		             OPENVPN_EDITOR_PLUGIN_ERROR,
-		             OPENVPN_EDITOR_PLUGIN_ERROR_FILE_NOT_OPENVPN,
+		             NMV_EDITOR_PLUGIN_ERROR,
+		             NMV_EDITOR_PLUGIN_ERROR_FILE_NOT_VPN,
 		             _("failed to write file: %s"),
 		             local->message);
 		return FALSE;
