@@ -18,26 +18,23 @@
  *
  */
 
-#include "config.h"
+#include "nm-default.h"
 
 #include <string.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <locale.h>
+#include <sys/stat.h>
 
-#include "nm-default.h"
-
-#include "nm-openvpn.h"
-#include "nm-openvpn-service-defines.h"
+#include "nm-openvpn-editor-plugin.h"
+#include "nm-openvpn-editor.h"
 #include "import-export.h"
+#include "utils.h"
 
-#include "nm-test-utils.h"
+#include "nm-utils/nm-test-utils.h"
 
-#define TEST_SRCDIR_CONF     TEST_SRCDIR"/conf"
-#define TEST_BUILDDIR_CONF   TEST_BUILDDIR"/conf"
-
-#define SRCDIR TEST_SRCDIR_CONF
-#define TMPDIR TEST_BUILDDIR_CONF
+#define SRCDIR TEST_SRCDIR"/conf"
+#define TMPDIR TEST_BUILDDIR"/conf-tmp"
 
 /*****************************************************************************/
 
@@ -77,23 +74,21 @@ get_basic_connection (NMVpnEditorPlugin *plugin,
 	return connection;
 }
 
-static void
-_check_item (NMSettingVpn *s_vpn,
-             const char *item,
-             const char *expected)
-{
-	g_assert (s_vpn);
-	g_assert_cmpstr (nm_setting_vpn_get_data_item (s_vpn, item), ==, expected);
-}
+#define _check_item(s_vpn, item, expected) \
+	G_STMT_START { \
+		NMSettingVpn *_s_vpn = (s_vpn); \
+		\
+		g_assert (_s_vpn); \
+		g_assert_cmpstr (nm_setting_vpn_get_data_item (_s_vpn, (item)), ==, (expected)); \
+	} G_STMT_END
 
-static void
-_check_secret (NMSettingVpn *s_vpn,
-               const char *item,
-               const char *expected)
-{
-	g_assert (s_vpn);
-	g_assert_cmpstr (nm_setting_vpn_get_secret (s_vpn, item), ==, expected);
-}
+#define _check_secret(s_vpn, item, expected) \
+	G_STMT_START { \
+		NMSettingVpn *_s_vpn = (s_vpn); \
+		\
+		g_assert (_s_vpn); \
+		g_assert_cmpstr (nm_setting_vpn_get_secret (_s_vpn, (item)), ==, (expected)); \
+	} G_STMT_END
 
 /*****************************************************************************/
 
@@ -237,7 +232,7 @@ test_tls_import (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE, NM_OPENVPN_CONTYPE_TLS);
 	_check_item (s_vpn, NM_OPENVPN_KEY_DEV, "tun");
 	_check_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, NULL);
-	_check_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, "yes");
+	_check_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, "adaptive");
 	_check_item (s_vpn, NM_OPENVPN_KEY_FLOAT, "yes");
 	_check_item (s_vpn, NM_OPENVPN_KEY_RENEG_SECONDS, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE, "173.8.149.245:1194");
@@ -330,7 +325,7 @@ test_tls_inline_import (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE, NM_OPENVPN_CONTYPE_TLS);
 	_check_item (s_vpn, NM_OPENVPN_KEY_DEV, "tun");
 	_check_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, NULL);
-	_check_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, "yes");
+	_check_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, "adaptive");
 	_check_item (s_vpn, NM_OPENVPN_KEY_FLOAT, "yes");
 	_check_item (s_vpn, NM_OPENVPN_KEY_RENEG_SECONDS, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE, "173.8.149.245:1194");
@@ -424,7 +419,7 @@ test_pkcs12_import (void)
 	_check_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE, NM_OPENVPN_CONTYPE_TLS);
 	_check_item (s_vpn, NM_OPENVPN_KEY_DEV, "tun");
 	_check_item (s_vpn, NM_OPENVPN_KEY_PROTO_TCP, NULL);
-	_check_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, "yes");
+	_check_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO, "adaptive");
 	_check_item (s_vpn, NM_OPENVPN_KEY_FLOAT, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_RENEG_SECONDS, NULL);
 	_check_item (s_vpn, NM_OPENVPN_KEY_REMOTE, "173.8.149.245:1194");
@@ -497,7 +492,6 @@ test_non_utf8_import (void)
 	NMConnection *connection;
 	NMSettingConnection *s_con;
 	NMSettingVpn *s_vpn;
-	const char *expected_cacert = "Attätaenko.pem";
 	char *expected_path;
 	const char *charset = NULL;
 
@@ -518,7 +512,7 @@ test_non_utf8_import (void)
 	s_vpn = nm_connection_get_setting_vpn (connection);
 	g_assert (s_vpn);
 
-	expected_path = g_strdup_printf ("%s/%s", SRCDIR, expected_cacert);
+	expected_path = g_strdup_printf ("%s/%s", SRCDIR, "Att\\344taenko.pem");
 	_check_item (s_vpn, NM_OPENVPN_KEY_CA, expected_path);
 	g_free (expected_path);
 
@@ -1117,7 +1111,7 @@ test_route_import (void)
 	/* IP4 setting */
 	s_ip4 = nm_connection_get_setting_ip4_config (connection);
 	g_assert (s_ip4);
-#ifdef NM_OPENVPN_OLD
+#ifdef NM_VPN_OLD
 	{
 		NMIP4Route *route;
 		const char *expected_nh2   = "0.0.0.0";
@@ -1293,9 +1287,9 @@ test_args_parse_line (void)
 	do_test_args_parse_line (" b \\ \\a ", TRUE, "b", " a");
 	do_test_args_parse_line ("\\ b \\ \\a ", TRUE, " b", " a");
 	do_test_args_parse_line ("'\\ b \\ \\a '", TRUE, "\\ b \\ \\a ");
-	do_test_args_parse_line ("\"\\ b \\ \\a \"a'b'", TRUE, " b  a ab");
-	do_test_args_parse_line ("\"\\ b \\ \\a \"a\\ 'b'", TRUE, " b  a a b");
-	do_test_args_parse_line ("\"\\ b \\ \\a \"a\\ 'b'   sd\\ \t", TRUE, " b  a a b", "sd ");
+	do_test_args_parse_line ("\"\\ b \\ \\a \"a'b'", TRUE, " b  a ", "a'b'");
+	do_test_args_parse_line ("\"\\ b \\ \\a \"a\\ 'b'", TRUE, " b  a ", "a 'b'");
+	do_test_args_parse_line ("\"\\ b \\ \\a \"a\\ 'b'   sd\\ \t", TRUE, " b  a ", "a 'b'", "sd ");
 
 	do_test_args_parse_line ("\"adfdaf  adf  ", FALSE);
 	do_test_args_parse_line ("\"adfdaf  adf  \\\"", FALSE);
@@ -1304,14 +1298,97 @@ test_args_parse_line (void)
 
 /*****************************************************************************/
 
+static void
+do_test_utils_str_utf8safe (const char *str, const char *expected)
+{
+	const char *str_safe, *s;
+	gs_free char *str2 = NULL;
+	gs_free char *str3 = NULL;
+
+	str_safe = nmv_utils_str_utf8safe_escape_c (str, &str2);
+
+	str3 = nmv_utils_str_utf8safe_escape (str);
+	g_assert_cmpstr (str3, ==, str_safe);
+	g_assert ((!str && !str3) || (str != str3));
+	g_clear_pointer (&str3, g_free);
+
+	if (expected == NULL) {
+		g_assert (str_safe == str);
+		g_assert (!str2);
+		if (str) {
+			g_assert (!strchr (str, '\\'));
+			g_assert (g_utf8_validate (str, -1, NULL));
+		}
+
+		g_assert (str == nmv_utils_str_utf8safe_unescape_c (str_safe, &str3));
+		g_assert (!str3);
+
+		str3 = nmv_utils_str_utf8safe_unescape (str_safe);
+		if (str) {
+			g_assert (str3 != str);
+			g_assert_cmpstr (str3, ==, str);
+		} else
+			g_assert (!str3);
+		g_clear_pointer (&str3, g_free);
+		return;
+	}
+
+	g_assert (str);
+	g_assert (str_safe != str);
+	g_assert (str_safe == str2);
+	g_assert (strchr (str, '\\') || !g_utf8_validate (str, -1, NULL));
+	g_assert (g_utf8_validate (str_safe, -1, NULL));
+
+	str3 = g_strcompress (str_safe);
+	g_assert_cmpstr (str, ==, str3);
+	g_clear_pointer (&str3, g_free);
+
+	str3 = nmv_utils_str_utf8safe_unescape (str_safe);
+	g_assert (str3 != str);
+	g_assert_cmpstr (str3, ==, str);
+	g_clear_pointer (&str3, g_free);
+
+	s = nmv_utils_str_utf8safe_unescape_c (str_safe, &str3);
+	g_assert (str3 != str);
+	g_assert (s == str3);
+	g_assert_cmpstr (str3, ==, str);
+	g_clear_pointer (&str3, g_free);
+
+	g_assert_cmpstr (str_safe, ==, expected);
+}
+
+static void
+test_utils_str_utf8safe (void)
+{
+	do_test_utils_str_utf8safe (NULL, NULL);
+	do_test_utils_str_utf8safe ("", NULL);
+	do_test_utils_str_utf8safe ("a", NULL);
+	do_test_utils_str_utf8safe ("ab", NULL);
+	do_test_utils_str_utf8safe ("abäb", NULL);
+	do_test_utils_str_utf8safe ("㈞abä㈞b", NULL);
+	do_test_utils_str_utf8safe ("Äab\\äb", "Äab\\\\äb");
+	do_test_utils_str_utf8safe ("ÄÄab\\äb", "ÄÄab\\\\äb");
+	do_test_utils_str_utf8safe ("Ä\304ab\\äb", "Ä\\304ab\\\\äb");
+}
+
+/*****************************************************************************/
+
 NMTST_DEFINE ();
 
 int main (int argc, char **argv)
 {
+	int errsv, result;
+
 	_nmovpn_test_temp_path = TMPDIR;
 
 	nmtst_init (&argc, &argv, TRUE);
 
+	if (mkdir (TMPDIR, 0755) != 0) {
+		errsv = errno;
+		if (errsv != EEXIST)
+			g_error ("failed creating \"%s\": %s", TMPDIR, g_strerror (errsv));
+	}
+
 #define _add_test_func_simple(func)       g_test_add_func ("/ovpn/properties/" #func, func)
 #define _add_test_func(detail, func, ...) nmtst_add_test_func ("/ovpn/properties/" detail, func, ##__VA_ARGS__)
 
@@ -1370,6 +1447,17 @@ int main (int argc, char **argv)
 
 	_add_test_func_simple (test_args_parse_line);
 
-	return g_test_run ();
+	_add_test_func_simple (test_utils_str_utf8safe);
+
+	result = g_test_run ();
+	if (result != EXIT_SUCCESS)
+		return result;
+
+	if (rmdir (TMPDIR) != 0) {
+		errsv = errno;
+		g_error ("failed deleting %s: %s", TMPDIR, g_strerror (errsv));
+	}
+
+	return EXIT_SUCCESS;
 }
 
